<?php
/**
 * @package Extension iubenda Cookie Law Solution for Joomla! greater than 2.5
 * @version 1.3.5
 * @author iubenda
 * @copyright (C) 2019-2020 iubenda
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/

defined( '_JEXEC' ) || die();

jimport( 'joomla.plugin.plugin' );
jimport( 'joomla.language.language' );
jimport( 'joomla.form.formfield' );

abstract class IubendaHelper {
	public static function render_tag_types( $name, $type, $selected ) {
		$html = '<select name="' . $name . '[custom_' . $type . 's][type][]">';

		foreach ( array(
			0 => 'Not set',
			1 => 'Strictly necessary',
			2 => 'Basic interactions & functionalities',
			3 => 'Experience enhancement',
			4 => 'Analytics',
			5 => 'Targeting & Advertising'
		) as $tag_id => $tag_name ) {
			$html .= '<option value="' . $tag_id . '"' . ( $selected === $tag_id ? ' selected="selected"' : '' ) . '>' . $tag_name . '</option>';
		}

		return $html . '</select>';
	}

	public static function get_custom_data( $data, $type ) {
		if ( ! is_array( $data ) )
			$data = (array) $data;

		// scripts
		if ( ! empty( $data ) && ! empty( $data[$type] ) && ! empty( $data['type'] ) ) {
			$filtered = array();

			// first field is template
			if ( count( $data[$type] ) > 1 ) {
				foreach ( $data[$type] as $number => $value ) {
					$trimmed = trim( $value );

					if ( $trimmed !== '' )
						$filtered[$trimmed] = (int) $data['type'][$number];
				}
			}

			$data = $filtered;
		} else
			$data = array();

		return $data;
	}

	public static function prepare_custom_data( $data ) {
		$newdata = array();

		foreach ( $data as $script => $type ) {
			if ( ! array_key_exists( $type, $newdata ) )
				$newdata[$type] = array();

			$newdata[$type][] = $script;
		}

		return $newdata;
	}
}

class JFormFieldJSCode extends JFormField {
	protected $type = 'JSCode';

	public function getInput() {
		$languages = JLanguage::getKnownLanguages();
		$field = '';

		foreach( $languages as $lang_id => $language ) {
			$value = isset( $this->value[$lang_id] ) ? $this->value[$lang_id] : '';
			$field .= '<div><label for="' . $this->id . '-' . $lang_id . '"></label><textarea name="' . $this->name . '[' . $lang_id . ']" id="' . $this->id . '-' . $lang_id . '" class="iubenda-textarea" style="width: 530px; height: 210px;">' . htmlentities( $value ) . '</textarea><p style="margin: 10px 0;">' . $language['name'] . '</p></div>';
		}

		return $field;
	}
}

class JFormFieldIubendaCustomScripts extends JFormField {
	protected $type = 'IubendaCustomScripts';

	public function getInput() {
		$plugin = JPluginHelper::getPlugin( 'system', 'iubenda' );
		$params = new JRegistry( $plugin->params );

		JHtml::_( 'script', JUri::root() . 'media/plg_system_iubenda/js/admin.js' );

		$param = $params->get( 'iubenda_custom_scripts', array() );
		$custom_scripts = array();

		// compatibility mode
		if ( is_string( $param ) && ! empty( $param ) ) {
			$scripts = array_filter( array_map( 'trim', explode( "\n", str_replace( "\r", '', $param ) ) ) );

			foreach ( $scripts as $script ) {
				$custom_scripts[$script] = 0;
			}
		} else if ( ! empty( $param->custom_scripts ) )
			$custom_scripts = IubendaHelper::get_custom_data( $param->custom_scripts, 'script' );

		$field = '
		<div id="tab-custom-scripts">
			<p class="description">Provide a list of custom scripts you’d like to block and assign their purpose.</p>
			<div id="custom-script-field-template" class="template-field" style="display: none;">
				<input type="text" value="" name="' . $this->name . '[custom_scripts][script][]" placeholder="Enter custom script" /> ' . IubendaHelper::render_tag_types( $this->name, 'script', 0 ) . ' <button class="btn btn-small button-cancel remove-custom-script-field"><span class="icon-cancel" aria-hidden="true"></span>Remove</button>
			</div>';

		if ( ! empty( $custom_scripts ) ) {
			foreach ( $custom_scripts as $script => $type ) {
				$field .= '
			<div class="custom-script-field">
				<input type="text" value="' . $script . '" name="' . $this->name . '[custom_scripts][script][]" placeholder="Enter custom script" /> ' . IubendaHelper::render_tag_types( $this->name, 'script', $type ) . ' <button class="btn btn-small button-cancel remove-custom-script-field"><span class="icon-cancel" aria-hidden="true"></span>Remove</button>
			</div>';
			}
		}

		$field .= '
			<a href="#" class="add-custom-script-field">Add New Script</a>
		</div>';

		return $field;
	}
}

class JFormFieldIubendaCustomIframes extends JFormField {
	protected $type = 'IubendaCustomIframes';

	public function getInput() {
		$plugin = JPluginHelper::getPlugin( 'system', 'iubenda' );
		$params = new JRegistry( $plugin->params );
		$param = $params->get( 'iubenda_custom_iframes', array() );
		$custom_iframes = array();

		// compatibility mode
		if ( is_string( $param ) && ! empty( $param ) ) {
			$iframes = array_filter( array_map( 'trim', explode( "\n", str_replace( "\r", '', $param ) ) ) );

			foreach ( $iframes as $iframe ) {
				$custom_iframes[$iframe] = 0;
			}
		} else if ( ! empty( $param->custom_iframes ) )
			$custom_iframes = IubendaHelper::get_custom_data( $param->custom_iframes, 'iframe' );

		$field = '
		<div id="tab-custom-iframes">
			<p class="description">Provide a list of custom iframes you’d like to block and assign their purpose.</p>
			<div id="custom-iframe-field-template" class="template-field" style="display: none;">
				<input type="text" value="" name="' . $this->name . '[custom_iframes][iframe][]" placeholder="Enter custom iframe" /> ' . IubendaHelper::render_tag_types( $this->name, 'iframe', 0 ) . ' <button class="btn btn-small button-cancel remove-custom-iframe-field"><span class="icon-cancel" aria-hidden="true"></span>Remove</button>
			</div>';

		if ( ! empty( $custom_iframes ) ) {
			foreach ( $custom_iframes as $iframe => $type ) {
				$field .= '
			<div class="custom-iframe-field">
				<input type="text" value="' . $iframe . '" name="' . $this->name . '[custom_iframes][iframe][]" placeholder="Enter custom iframe" /> ' . IubendaHelper::render_tag_types( $this->name, 'iframe', $type ) . ' <button class="btn btn-small button-cancel remove-custom-iframe-field"><span class="icon-cancel" aria-hidden="true"></span>Remove</button>
			</div>';
			}
		}

		$field .= '
			<a href="#" class="add-custom-iframe-field">Add New Iframe</a>
		</div>';

		return $field;
	}
}

class PlgSystemIubenda extends JPlugin {
	function __construct( & $subject, $config ) {
		parent::__construct( $subject, $config );

		$this->loadLanguage();
	}

	function onAfterRender() {
		// passing this parameter on a website will disactivate the iubenda script and therefore avoid all parsing/replacing operations
		if ( isset( $_GET[ 'iub_no_parse' ] ) || isset( $_SERVER['HTTP_X_REQUESTED_WITH'] ) )
			return true;

		$app = JFactory::getApplication();

		// front page?
		if ( $app->isSite() ) {
			require( dirname( __FILE__ ) . '/iubenda-cookie-class/iubenda.class.php' );

			$no_parse = (bool) $this->params->get( 'iubenda_no_parse' );

			// get langauge data
			$lang = JFactory::getLanguage();
			$current_lang = $lang->getTag();
			$codes = $this->params->get( 'iubenda_code' );

			// check current language
			if ( isset( $codes->$current_lang ) )
				$iub_code = $codes->$current_lang;
			// check any language
			elseif ( ! empty( $codes ) )
				$iub_code = reset( $codes );
			// nothing?
			else
				$iub_code = '';

			if ( preg_match( '/(?:"|\')perPurposeConsent(?:"|\')\: *(?:"|\'*)true(?:"|\'*)/', $iub_code ) === 1 )
				$no_parse = false;

			// check whether to run parser or not
			if ( ( $no_parse && iubendaParser::consent_given() ) || iubendaParser::bot_detected() )
				return true;

			// bail on POST request
			if ( $_POST && (bool) $this->params->get( 'iubenda_output_post' ) )
				return true;

			// including iubenda class and simpleHTMLDom
			$content = JResponse::getBody();

			if ( strpos( $content, '<html' ) === false || strpos( $content, '<html' ) > 200 )
				return true;

			$iub_code .="\n
			<script type='text/javascript'>
				var iCallback = function(){};

				if('callback' in _iub.csConfiguration) {
					if('onConsentGiven' in _iub.csConfiguration.callback) iCallback = _iub.csConfiguration.callback.onConsentGiven;

					_iub.csConfiguration.callback.onConsentGiven = function()
					{
						iCallback();

						jQuery('noscript._no_script_iub').each(function (a, b) { var el = jQuery(b); el.after(el.html()); });
					};
				};
			</script>";

			$scripts = $this->params->get( 'iubenda_custom_scripts' );

			// compatibility mode
			if ( is_string( $scripts ) && ! empty( $scripts ) ) {
				$new_scripts = array_filter( array_map( 'trim', explode( "\n", str_replace( "\r", '', $scripts ) ) ) );

				foreach ( $new_scripts as $script ) {
					$custom_scripts[$script] = 0;
				}
			} else if ( ! empty( $scripts->custom_scripts ) )
				$custom_scripts = IubendaHelper::get_custom_data( $scripts->custom_scripts, 'script' );
			else
				$custom_scripts = array();

			$iframes = $this->params->get( 'iubenda_custom_iframes' );

			// compatibility mode
			if ( is_string( $iframes ) && ! empty( $iframes ) ) {
				$new_iframes = array_filter( array_map( 'trim', explode( "\n", str_replace( "\r", '', $iframes ) ) ) );

				foreach ( $new_iframes as $iframe ) {
					$custom_iframes[$iframe] = 0;
				}
			} else if ( ! empty( $iframes->custom_iframes ) )
				$custom_iframes = IubendaHelper::get_custom_data( $iframes->custom_iframes, 'iframe' );
			else
				$custom_iframes = array();

			$custom_scripts = IubendaHelper::prepare_custom_data( $custom_scripts );
			$custom_iframes = IubendaHelper::prepare_custom_data( $custom_iframes );

			if ( (int) $this->params->get( 'iubenda_experimental_parser' ) === 1 ) {
				$iubenda = new iubendaParser( mb_convert_encoding( $content, 'HTML-ENTITIES', 'UTF-8' ), array( 'type' => 'faster', 'scripts' => $custom_scripts, 'iframes' => $custom_iframes ) );

				// render output
				$content = $iubenda->parse();

				$content .= "<!-- IUBENDA PRIMARY PARSING ENGINE -->";
			} else {
				$iubenda = new iubendaParser( $content, array( 'type' => 'page', 'scripts' => $custom_scripts, 'iframes' => $custom_iframes ) );

				// render output
				$content = $iubenda->parse();

				$content .= "<!-- IUBENDA SECONDARY PARSING ENGINE -->";
			}

			$content = str_replace( "<head>", "<head>" . $iub_code, $content );

			JResponse::setBody( $content."<!-- IUBENDA PLUGIN ACTIVE -->" );
		}

		return true;
	}
}
